# Copyright (c) 2011, Intel Corporation
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright notice,
#       this list of conditions and the following disclaimer in the documentation
#       and/or other materials provided with the distribution.
#     * Neither the name of Intel Corporation nor the names of its contributors
#       may be used to endorse or promote products derived from this software
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""redirect module."""

import cStringIO as _cStringIO
import acpi as _acpi
import bits as _bits
import struct as _struct
import sys as _sys

class Tee(object):
    """Tee output to both input files provided."""
    def __init__(self, out1, out2):
        self.out1 = out1
        self.out2 = out2
    def write(self, data):
        self.out1.write(data)
        self.out2.write(data)

# Create the store for the log file
_log = _cStringIO.StringIO()

def _log_open():
    return _log.tell()

def _log_read(offset, size):
    return _log.getvalue()[offset:offset+size]

def _acpilog_open():
    return _acpi.TableHeader.size + _log_open()

_acpilog_header = _acpi.TableHeader(signature="BITS", length=0, revision=0, checksum=0, oemid="INTEL\0", oemtableid="00000000", oemrevision=0, creatorid="INTL", creatorrevision=0)

def _acpilog_read(offset, size):
    logstr = _log.getvalue()
    if offset >= _acpi.TableHeader.size:
        return logstr[(offset - _acpi.TableHeader.size) : (offset - _acpi.TableHeader.size + size)]
    logstr_sum = sum(ord(c) for c in logstr)
    header = _acpilog_header._replace(length=_acpilog_open())
    header_sum = sum(ord(c) for c in header.repack())
    header = header._replace(checksum = 256 - (logstr_sum + header_sum) & 0xFF)
    return (header.repack() + logstr)[offset:offset+size]

def redirect():
    """Redirect all screen outputs (stdout and stderr) to a log file.

    Not to be called except as part of system initialization."""
    _sys.stdout = Tee(_sys.stdout, _log)
    _sys.stderr = Tee(_sys.stderr, _log)

    _bits.pyfs_add("log", _log_open, _log_read)
    _bits.pyfs_add("acpilog", _acpilog_open, _acpilog_read)

def clear():
    """Clear the log file."""
    _log.truncate(0)
